"""
The permissions() directive only accepts permission ids or permission classes:

  >>> import grok
  >>> grok.testing.grok('grok.tests.security.not_a_permission_class_fixture')
  Traceback (most recent call last):
  ...
  GrokImportError: You can only pass unicode values, ASCII values, or
  subclasses of grok.Permission to the 'permissions' directive.

"""



"""
  >>> import grok
  >>> from zope.configuration import xmlconfig
  >>> context = xmlconfig.file('meta.zcml', grok)

  >>> ignored = xmlconfig.string('''
  ... <configure
  ...     xmlns="http://namespaces.zope.org/zope"
  ...     xmlns:grok="http://namespaces.zope.org/grok"
  ...     >
  ...     <grok:grok package="grok.tests.zcml.stoneage.cave"/>
  ... </configure>''', context=context)

  >>> from grok.tests.zcml.stoneage.cave import Cave
  >>> cave = Cave()

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component

  >>> view = component.getMultiAdapter((cave, request), name='index')
  >>> print view()
  <html>
  <body>
  <h1>A comfy cave</h1>
  </body>
  </html>
"""



##############################################################################
#
# Copyright (c) 2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok test helpers.
"""
import sys
from zope.configuration.config import ConfigurationMachine
from grokcore.component import zcml
# Provide this import here for BBB reasons:
from grokcore.component.testing import grok_component


def grok(module_name=None):
    """Grok a module.

    Test helper to 'grok' a module named by `module_name`, a dotted
    path to a module like ``'mypkg.mymodule'``. 'grokking' hereby
    means to do all the ZCML configurations triggered by directives
    like ``grok.context()`` etc. This is only needed if your module
    was not `grokked` during test setup time as it normally happens
    with functional tests.
    """
    config = ConfigurationMachine()
    zcml.do_grok('grokcore.component.meta', config)
    zcml.do_grok('grokcore.security.meta', config)
    zcml.do_grok('grokcore.view.meta', config)
    zcml.do_grok('grokcore.view.templatereg', config)
    zcml.do_grok('grokcore.viewlet.meta', config)
    zcml.do_grok('grokcore.formlib.meta', config)
    zcml.do_grok('grokcore.annotation.meta', config)
    zcml.do_grok('grokcore.site.meta', config)
    zcml.do_grok('grokcore.catalog.meta', config)
    zcml.do_grok('grokcore.traverser.meta', config)
    zcml.do_grok('grokcore.rest.meta', config)
    zcml.do_grok('grokcore.xmlrpc.meta', config)
    if module_name is not None:
        zcml.do_grok(module_name, config)
    config.execute_actions()


def warn(message, category=None, stacklevel=1):
    """Intended to replace warnings.warn in tests.

    Modified copy from zope.deprecation.tests to:

      * make the signature identical to warnings.warn
      * to check for \*.pyc and \*.pyo files.

    When zope.deprecation is fixed, this warn function can be removed again.
    """
    print "From grok.testing's warn():"

    frame = sys._getframe(stacklevel)
    path = frame.f_globals['__file__']
    if path.endswith('.pyc') or path.endswith('.pyo'):
        path = path[:-1]

    file = open(path)
    lineno = frame.f_lineno
    for i in range(lineno):
        line = file.readline()

    print "%s:%s: %s: %s\n  %s" % (
        path,
        frame.f_lineno,
        category.__name__,
        message,
        line.strip(),
        )



"""
Subclasses of grok.GlobalUtility must implement exactly one interface:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: <class 'grok.tests.utility.implementsnone2.Club'> must
  implement at least one interface (use grok.implements to specify).
"""
import grok

class Club(object):
    pass

grok.global_utility(Club)



"""
A Role component optionally defines what permission it comprises.

The grok.permissions() directive is used to specify the set of permissions
that are aggregated in the particular Role. The permissions can be referenced
by "name" or by class.

  >>> grok.testing.grok(__name__)
"""

import grok
import zope.interface

class FirstPermission(grok.Permission):
    grok.name('first permission')

class SecondPermission(grok.Permission):
    grok.name('second permission')

class RoleComprisingTwoPermissionsByName(grok.Role):
    grok.name('ByName')
    grok.permissions(
        'first permission',
        'second permission'
        )

class RoleComprisingTwoPermissionsByClass(grok.Role):
    grok.name('ByClass')
    grok.permissions(
        FirstPermission,
        SecondPermission
        )



"""

Forms have an application_url() method to easily retrieve the url of the
application, like views does::

  >>> getRootFolder()['world'] = world = IceWorld()
  >>> world['arthur'] = Mammoth()

And we can access the display form which display the application URL::

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open('http://localhost/world/arthur')
  >>> print browser.contents
  <p> Test display: application http://localhost/world </p>

Same for the edit form::

  >>> browser.open('http://localhost/world/arthur/@@edit')
  >>> print browser.contents
  <p> Test edit: application http://localhost/world </p>


"""
import grok
from zope import schema

class IceWorld(grok.Application, grok.Container):
    pass


class Mammoth(grok.Model):
    class fields:
        name = schema.TextLine(title=u"Name")
        size = schema.TextLine(title=u"Size", default=u"Quite normal")


class Index(grok.DisplayForm):

    grok.context(Mammoth)

index = grok.PageTemplate("""
<p>
   Test display: application <tal:replace tal:replace="view/application_url" />
</p>""")


class Edit(grok.EditForm):
    
    grok.context(Mammoth)

edit = grok.PageTemplate("""
<p>
   Test edit: application <tal:replace tal:replace="view/application_url" />
</p>""")



"""
Demonstrate the grokcore.chameleon template component has been registered.


  >>> from zope.app.wsgi.testlayer import Browser
  >>> getRootFolder()["mammoth"] = Mammoth()
  >>> browser = Browser()
  >>> browser.open("http://localhost/mammoth/@@index")
  >>> print browser.contents
  <html>Mammoth</html>

"""
import grok

grok.templatedir('templates')

class Mammoth(grok.Model):
    pass

class Index(grok.View):
    grok.context(Mammoth)
    grok.template('available')



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""The grok demo wiki
"""
import re
import grok

from z3c.flashmessage.interfaces import IMessageReceiver
from zope import component

LINK_PATTERN = re.compile('\[\[(.*?)\]\]')
find_wiki_links = LINK_PATTERN.findall


class WikiPage(grok.Model):

    def __init__(self):
        self.text = u"GROK EMPTY WIKI PAGE. FILL!"

    def update(self, text):
        links = find_wiki_links(text)
        for link in links:
            if link not in self.__parent__:
                self.__parent__[link] = WikiPage()
        self.text = text


class Layout(grok.View):
    pass


class Index(grok.View):

    def update(self):
        wiki_url = self.url(self.context.__parent__)
        self.rendered_text, replacements = (
            LINK_PATTERN.subn(r'<a href="%s/\1">\1</a>' % wiki_url, 
                              self.context.text))

class Edit(grok.View):

    def update(self):
        text = self.request.form.get('wikidata')
        if not text:
            return # Just render the template

        # Update the text and redirect
        self.context.update(text)
        self.flash('Saved.')
        self.redirect(self.url(self.context))

class Messages(grok.View):
    @property
    def messages(self):
        receiver = component.getUtility(IMessageReceiver)
        return receiver.receive()

class WikiLayer(grok.IRESTLayer):
    # This skin will be visible as http://localhost:8080/++rest++wiki/...
    grok.restskin('wiki')

class PageRest(grok.REST):
    grok.layer(WikiLayer)
    
    def GET(self):
        return "Hello world"



"""

Permissions already set by non-grok components are preserved by the
Grok publisher.

Let's first define a ``@@contents.html`` that is protected by a Zope
permission, ``zope.ManageContent``::

  >>> from zope.publisher.browser import BrowserPage
  >>> class Contents(BrowserPage):
  ...   def __init__(self, context, request):
  ...     self.context = context
  ...     self.request = request
  ...   def __call__(self):
  ...     return "Contents called"
  >>> from zope import component
  >>> from zope.interface import Interface
  >>> from zope.publisher.interfaces.browser import IBrowserRequest
  >>> component.provideAdapter(Contents,
  ...   adapts=(Interface, IBrowserRequest),
  ...   provides=Interface,
  ...   name='contents.html')
  >>> from zope.security.checker import Checker, defineChecker
  >>> required = {}
  >>> required['__call__'] = 'zope.ManageContent'
  >>> required['browserDefault'] = 'zope.ManageContent'
  >>> defineChecker(Contents, Checker(required))

The `@@contents.html` view of folders is protected by
`zope.ManageContent` and should not be visible to unauthenticated
users. Instead we are asked to authenticate ourselves::

  >>> print http('GET /@@contents.html HTTP/1.1')
  HTTP/1.0 401 Unauthorized
  ...

Let's test this in the context of a Grok application:

  >>> grok.testing.grok(__name__)
  >>> from grok.ftests.security.preserve_permissions import App
  >>> root = getRootFolder()
  >>> root['app'] = App()

Now there is a ``contents.html`` view available for our application,
which is protected by default::

  >>> print http('GET /app/@@contents.html HTTP/1.1')
  HTTP/1.0 401 Unauthorized
  ...

However, if we make a grant, e.g. on the root object, we can access
the view just fine:

  >>> from zope.securitypolicy.interfaces import IPrincipalPermissionManager
  >>> root = getRootFolder()
  >>> root_perms = IPrincipalPermissionManager(root)
  >>> root_perms.grantPermissionToPrincipal('zope.ManageContent',
  ...                                       'zope.anybody')
  >>> print http('GET /@@contents.html HTTP/1.1')
  HTTP/1.0 200 Ok
  ...

The default view is accessible::

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()
  >>> browser.open('http://localhost/app')
  >>> print browser.contents
  Moo!

While the manage view is locked::

  >>> browser.open('http://localhost/app/@@manage')
  Traceback (most recent call last):
  ...
  httperror_seek_wrapper: HTTP Error 401: Unauthorized

When we authenticate, everything works fine::

  >>> browser.addHeader('Authorization', 'Basic mgr:mgrpw')
  >>> browser.open('http://localhost/app/@@manage')
  >>> print browser.contents
  Woo!

"""
import grok

class ManageApp(grok.Permission):
    grok.name('app.Manage')

class App(grok.Application, grok.Container):
    pass

class Index(grok.View):
    def render(self):
        return 'Moo!'

class Manage(grok.View):
    grok.require('app.Manage')
    def render(self):
        return 'Woo!'



"""
The baseclass directive can be used to mark something a base class. Of course
subclasses shouldn't inherit this otherwise there is no way to turn them
into non-base classes.

  >>> grok.testing.grok(__name__)

  >>> from zope.publisher.browser import TestRequest
  >>> request = TestRequest()
  >>> from zope import component
  >>> model = Model()

We can't look up SomeView as a view, as it's a base class:

  >>> view = component.getMultiAdapter((model, request), name='someview')
  Traceback (most recent call last):
    ...
  ComponentLookupError: ((<grok.tests.baseclass.basedirective.Model object at 0x...>,
  <zope.publisher.browser.TestRequest instance ...>),
  <InterfaceClass zope.interface.Interface>,
  'someview')

We can however get a subclass of SomeView:

  >>> view = component.getMultiAdapter((model, request), name='anotherview')
  >>> view.render()
  'hello world'
"""
import grok

class Model(grok.Model):
    pass

class SomeView(grok.View):
    grok.baseclass()
    
    def render(self):
        return "hello world"

class AnotherView(SomeView):
    pass




##############################################################################
#
# Copyright (c) 2006-2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Events for Grok application components.

The events described here are *not* trigged by Grok itself. They are
conveniently provided to be used in your own application.

"""
import grok
import grokcore.site.interfaces
from zope.interface import implements


class ApplicationInitializedEvent(object):
    """A Grok Application has been created and is now ready to be used.
    """
    implements(grok.IApplicationInitializedEvent)

    def __init__(self, app):
        assert grok.IApplication.providedBy(app)
        self.object = app



from setuptools import setup, find_packages

setup(
    name='grokwiki',
    version='0.1',
    author='Grok Team',
    author_email='grok-dev@zope.org',
    url='http://svn.zope.org/grok/trunk',
    description="""\
Grok: Now even cavemen can use wikis!
""",
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data = True,
    zip_safe=False,
    license='ZPL',

    install_requires=['setuptools',
                      'grok',
                      'grokui.admin',
                      'grokcore.startup',
                      'grokcore.message',
                      ],
    entry_points = """
      [console_scripts]
      grokwiki-debug = grokcore.startup:interactive_debug_prompt
      grokwiki-ctl = grokcore.startup:zdaemon_controller
      [paste.app_factory]
      main = grokcore.startup:application_factory
      """,
)



"""
Imported model and adapter won't be grokked:

  >>> import grok
  >>> from grok.tests.adapter.adapter import IHome

  >>> grok.testing.grok(__name__)
  >>> cave = Cave()
  >>> home = IHome(cave)
  Traceback (most recent call last):
    ...
  TypeError: ('Could not adapt', <grok.tests.adapter.adapter.Cave object at ...>, <InterfaceClass grok.tests.adapter.adapter.IHome>)

"""
from grok.tests.adapter.adapter import Cave, Home



"""
Subclasses of grok.GlobalUtility that implement more than one interface must
specify which interface to use for the registration:

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: <class 'grok.tests.utility.implementsmany.Club'> is implementing
  more than one interface (use grok.provides to specify which one to use).
"""
import grok
from zope import interface

class IClub(interface.Interface):
    pass

class ISpikyClub(interface.Interface):
    pass

class Club(grok.GlobalUtility):
    grok.implements(IClub, ISpikyClub)



"""
A Role component have a title and description, that can be internationalized.

Let's grok this package and check we still have a Message object for the
internationalized title and description of the defined roles.

  >>> grok.testing.grok(__name__)
  >>> from zope.securitypolicy.interfaces import IRole
  >>> from zope.component import getUtility
  >>> from zope.i18nmessageid import Message

A grok.Role without any internationalization.
The id, title and description should be unicode::

  >>> role = getUtility(IRole, name="RoleWithoutI18n")
  >>> role.id
  u'RoleWithoutI18n'
  >>> role.title
  u'RoleWithoutI18n'
  >>> role.description
  u'My role without i18n'
  >>>
  >>> isinstance(role.id, Message)
  False
  >>> isinstance(role.title, Message)
  False
  >>> isinstance(role.description, Message)
  False

A grok.Role registered with the name and description directives only, both
internationalized.
The id is taken from the name directive and should not be a Message object.
The title is taken from the name directive because the title directive is not used.
::

  >>> role = getUtility(IRole, name="RoleWithI18n")
  >>> isinstance(role.id, Message)
  False
  >>> isinstance(role.title, Message)
  True
  >>> isinstance(role.description, Message)
  True

A grok.Role registered with name, title and description directives::

  >>> role = getUtility(IRole, name="RoleWithI18nTitle")
  >>> isinstance(role.id, Message)
  False
  >>> isinstance(role.title, Message)
  True
  >>> isinstance(role.description, Message)
  True
"""

import grok
import zope.interface
from zope.i18nmessageid import MessageFactory

_ = MessageFactory("testi18n")


class RoleWithoutI18n(grok.Role):
    grok.name('RoleWithoutI18n')
    grok.description('My role without i18n')


class RoleWithI18n(grok.Role):
    grok.name(_('RoleWithI18n'))
    grok.description(_(u'My role with i18n'))


class RoleWithI18nTitle(grok.Role):
    grok.name('RoleWithI18nTitle')
    grok.title(_('RoleWithI18n'))
    grok.description(_(u'My role with i18n'))



"""
Thanks to Zope's event system, newly added objects are automatically
catalogued, should a catalog be present.

  >>> getRootFolder()["zoo"] = Zoo()

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

Let's demonstrate that an object that has not been added to a
container yet can still be modified using a form's applyData method.
Event though this method triggers an IObjectModifiedEvent, the catalog
won't be bothered by this.  It will start the indexation when the
object has been *added* to a container, not before.

  >>> browser.open("http://localhost/zoo/@@addmammoth")
  >>> browser.getControl(name="form.name").value = "Ellie the Mammoth"
  >>> browser.getControl(name="form.size").value = "Really small"
  >>> browser.getControl("Add entry").click()

  >>> browser.open("http://localhost/zoo/ellie")
  >>> print browser.contents
  Hi, my name is Ellie the Mammoth, and I\'m "Really small"

Let's ensure the catalog has actually indexed the object with the
right value:

  >>> browser.open("http://localhost/zoo/search")
  >>> print browser.contents
  We found Ellie!

"""
import grok
from zope import schema, interface, component
from zope.intid import IntIds
from zope.intid.interfaces import IIntIds
from zope.catalog.catalog import Catalog
from zope.catalog.interfaces import ICatalog
from zope.catalog.field import FieldIndex

def setup_catalog(catalog):
    catalog['name'] = FieldIndex('name', IMammoth)

class Zoo(grok.Site, grok.Container):
    grok.local_utility(IntIds, provides=IIntIds)
    grok.local_utility(Catalog, provides=ICatalog, setup=setup_catalog)

class IMammoth(interface.Interface):
    name = schema.TextLine(title=u"Name")
    size = schema.TextLine(title=u"Size")

class Mammoth(grok.Model):
    grok.implements(IMammoth)

class Index(grok.View):
    grok.context(Mammoth)

    def render(self):
        return 'Hi, my name is %s, and I\'m "%s"' % (self.context.name,
                                                     self.context.size)

class Search(grok.View):
    grok.context(Zoo)

    def render(self):
        catalog = component.getUtility(ICatalog)
        query = ('Ellie the Mammoth', 'Ellie the Mammoth')
        results = catalog.searchResults(name=query)
        if len(list(results)) == 1:
            return 'We found Ellie!'
        return "Couldn't find Ellie."

class AddMammoth(grok.AddForm):
    grok.context(Zoo)

    form_fields = grok.AutoFields(IMammoth)

    @grok.action('Add entry')
    def add(self, **data):
        # First apply the form data, thus triggering an
        # IObjectModifiedEvent.  This test case demonstrates that this
        # isn't a problem.
        ellie = Mammoth()
        self.applyData(ellie, **data)
        self.context['ellie'] = ellie



"""
You can also specify interfaces instead of classes with
`grok.context` (class-level):

  >>> grok.testing.grok(__name__)

  >>> cave = Cave()
  >>> home = IHome(cave)

  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home)
  True

  >>> hole = Hole()
  >>> home = IHome(hole)

  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home)
  True

"""
import grok
from zope import interface

class ICave(interface.Interface):
    pass

class Cave(grok.Model):
    grok.implements(ICave)

class Hole(grok.Model):
    grok.implements(ICave)

class IHome(interface.Interface):
    pass

class Home(grok.Adapter):
    grok.implements(IHome)
    grok.context(ICave)



"""
==============================
Application lifecycle overview
==============================

The application initialization is an important moment of your
application's life. This creation process is responsible for the
local utilities and the catalog indexes' creation. Let's create a site
to demonstrate the different steps.

  >>> root = getRootFolder()

The life cycle of an application is made of 2 major events :
ObjectCreatedEvent and ObjectAddedEvent.

ObjectCreatedEvent shoud be triggered when an application is
instanciated. At this step, the application doesn't have a site
manager yet.
  
  >>> site = Herd()
  >>> notify(grok.ObjectCreatedEvent(site))
  >>> setSite(site)
  Traceback (most recent call last):
  ...
  ComponentLookupError: no site manager defined

As the application is instanciated and notified as "created", we can
now persist it. This operation will fire the "ObjectAddedEvent". This
event is trigged by the container and the associated event handlers
will create the site manager. Once the site manager is created, Grok's
subscribers will create the local utilities and the indexes.

This particular step is often used to trigger the creation of
content. However, as the event handlers are not ordered, we can not be
sure that our handler will be called after the local utilities
creation. This can be quite annoying : the content created by our
handler may not be regitered in the Catalog and the IntIds utility.

To demonstrate this behavior, we now persist the application. This
will trigger a test handler that queries the Catalog::

  >>> root['site'] = site
  Catalog can not be found !

Now our application is persisted, the site manager and the local
utilities are created. We can safely use our application. An event is
provided by Grok, to be fired at this step. The same test event shows
that we can now work on an operational environment:

  >>> notify(grok.ApplicationInitializedEvent(site))
  <zope.catalog.catalog.Catalog object at ...>

"""
import grok
from grok import index
from zope.event import notify
from zope.schema import TextLine
from zope.interface import Interface
from zope.component import queryUtility
from zope.site.hooks import setSite
from zope.catalog.interfaces import ICatalog


class Herd(grok.Container, grok.Application):
    pass


class IPachyderm(Interface):
    tusks = TextLine(title = u"About the tusks")


class TuskIndex(grok.Indexes):
    grok.context(IPachyderm)
    grok.site(Herd)

    tusks = index.Text()


@grok.subscribe(Herd, grok.IObjectAddedEvent)
@grok.subscribe(Herd, grok.IApplicationInitializedEvent)
def CatalogTester(application, event):
    catalog = queryUtility(ICatalog, context=application)
    if catalog is None:
        print "Catalog can not be found !"
    else:
        print catalog



##############################################################################
#
# Copyright (c) 2006-2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok utility functions.
"""
import grok
import grokcore.site.interfaces
import zope.location.location

from zope import interface
from zope.schema.interfaces import WrongType
from grokcore.view.util import url
from grokcore.site.util import getApplication


def safely_locate_maybe(obj, parent, name):
    """Set an object's ``__parent__`` (and ``__name__``) if the object's
    ``__parent__`` attribute doesn't exist yet or is ``None``.

    If the object provides ``ILocation``, ``__parent__`` and
    ``__name__`` will be set directly.  A location proxy will be
    returned otherwise.
    """
    if getattr(obj, '__parent__', None) is not None:
        return obj
    # This either sets __parent__ or wraps 'obj' in a LocationProxy
    return zope.location.location.located(obj, parent, name)


def applySkin(request, skin, skin_type):
    """Change the presentation skin for this request.
    """
    # Remove all existing skin declarations (commonly the default skin).
    ifaces = [iface for iface in interface.directlyProvidedBy(request)
              if not skin_type.providedBy(iface)]
    # Add the new skin.
    ifaces.append(skin)
    interface.directlyProvides(request, *ifaces)


def application_url(request, obj, name=None, data={}):
    """Return the URL of the nearest enclosing :class:`grok.Application`.

    Raises :exc:`ValueError` if no application can be found.
    """
    return url(request, getApplication(), name, data)


def create_application(factory, container, name):
    """Creates an application and triggers the events from
    the application lifecycle.
    """
    # Check the factory.
    if not grokcore.site.interfaces.IApplication.implementedBy(factory):
        raise WrongType(factory)

    # Check the availability of the name in the container.
    if name in container:
        raise KeyError(name)

    # Instanciate the application
    application = factory()

    # Trigger the creation event.
    grok.notify(grok.ObjectCreatedEvent(application))

    # Persist the application.
    # This may raise a KeyError.
    container[name] = application

    # Trigger the initialization event.
    grok.notify(grok.ApplicationInitializedEvent(application))

    return application



##############################################################################
#
# Copyright (c) 2006-2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok
"""
from zope.interface import implements
from zope.component import adapts

from martian import ClassGrokker, InstanceGrokker, GlobalGrokker
from martian import baseclass
from martian.error import GrokError, GrokImportError

from grokcore.component import Context
from grokcore.component import GlobalUtility
from grokcore.component import Adapter, MultiAdapter, GlobalUtility, Context
from grokcore.component import Subscription, MultiSubscription
from grokcore.component import querySubscriptions, queryMultiSubscriptions
from grokcore.component import queryOrderedSubscriptions
from grokcore.component import queryOrderedMultiSubscriptions

from grokcore.component.decorators import subscribe, adapter, implementer

from grokcore.component.directive import context, name, title, description
from grokcore.component.directive import provides, direct
from grokcore.component.directive import global_utility, global_adapter

from grokcore.content import Model, Container, OrderedContainer

from grokcore.security import Permission, Role
from grokcore.security import Public
from grokcore.security import require
from grokcore.security import permissions

from grokcore.view import ContentProvider
from grokcore.view import PageTemplate
from grokcore.view import PageTemplateFile
from grokcore.view import DirectoryResource
from grokcore.view import layer
from grokcore.view import template
from grokcore.view import templatedir
from grokcore.view import skin
from grokcore.view import url
from grokcore.view import path

from grokcore.viewlet import Viewlet
from grokcore.viewlet import ViewletManager
from grokcore.viewlet import view
from grokcore.viewlet import viewletmanager
from grokcore.viewlet import order

from grokcore.formlib import action
from grokcore.formlib import AutoFields
from grokcore.formlib import Fields

from grokcore.layout import UnauthorizedPage
from grokcore.layout import NotFoundPage
from grokcore.layout import ExceptionPage
from grokcore.layout import layout

from grokcore.annotation import Annotation

from grokcore.site import IApplication
from grokcore.site import IApplicationInitializedEvent
from grokcore.site import Application
from grokcore.site import ApplicationInitializedEvent
from grokcore.site import getApplication
from grokcore.site import getSite
from grokcore.site import local_utility
from grokcore.site import LocalUtility
from grokcore.site import site
from grokcore.site import Site

from zope.event import notify

from zope.lifecycleevent import IObjectCopiedEvent
from zope.lifecycleevent import IObjectCreatedEvent
from zope.lifecycleevent import IObjectModifiedEvent
from zope.lifecycleevent import ObjectCopiedEvent
from zope.lifecycleevent import ObjectCreatedEvent
from zope.lifecycleevent import ObjectModifiedEvent

from zope.app.publication.interfaces import IBeforeTraverseEvent

from zope.publisher.interfaces.browser import IBrowserRequest
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

from zope.container.interfaces import IContainerModifiedEvent
from zope.container.interfaces import IObjectAddedEvent
from zope.container.interfaces import IObjectMovedEvent
from zope.container.interfaces import IObjectRemovedEvent
from zope.container.contained import ContainerModifiedEvent
from zope.container.contained import ObjectAddedEvent
from zope.container.contained import ObjectMovedEvent
from zope.container.contained import ObjectRemovedEvent

from grok.components import AddForm
from grok.components import AddFormPage
from grok.components import DisplayForm
from grok.components import DisplayFormPage
from grok.components import EditForm
from grok.components import EditFormPage
from grok.components import ExceptionView
from grok.components import Form
from grok.components import FormPage
from grok.components import Layout
from grok.components import NotFoundView
from grok.components import Page
from grok.components import UnauthorizedView
from grok.components import View

from grokcore.json import JSON
from grokcore.xmlrpc import XMLRPC

from grokcore.catalog import Indexes
from grokcore.catalog import index

from grokcore.traverser import Traverser
from grokcore.traverser import traversable

from grokcore.rest import IRESTLayer
from grokcore.rest import IRESTSkinType
from grokcore.rest import REST
from grokcore.rest import restskin

# BBB These two functions are meant for test fixtures and should be
# imported from grok.testing, not from grok.
from grok.testing import grok, grok_component

# Our __init__ provides the grok API directly so using 'import grok' is enough.
from grok.interfaces import IGrokAPI
from zope.interface import moduleProvides
moduleProvides(IGrokAPI)
__all__ = list(IGrokAPI)



"""
Since grok.global_utility is a MultipleTimesDirective, there is a list of
GlobalUtilityInfo objects annotated on the module.

  >>> from grok.tests.directive import multipletimes
  >>> guis = grok.global_utility.bind().get(multipletimes)
  >>> len(guis)
  2

  >>> factory, provides, name, direct = guis[0]
  >>> factory
  <class 'grok.tests.directive.multipletimes.Club'>

  >>> provides
  <InterfaceClass grok.tests.directive.multipletimes.IClub>

  >>> name
  'foo'

  >>> factory, provides, name, direct = guis[1]
  >>> factory
  <class 'grok.tests.directive.multipletimes.Cave'>

  >>> provides is None
  True

  >>> name
  u''

"""
import grok
from zope import interface

class IClub(interface.Interface):
    pass

class ICave(interface.Interface):
    pass

class Club(object):
    grok.implements(IClub)

class Cave(object):
    grok.implements(ICave)

grok.global_utility(Club, provides=IClub, name='foo')
grok.global_utility(Cave)



"""
Global utilities can be created by subclassing grok.GlobalUtility:

  >>> grok.testing.grok(__name__)
  >>> from zope import component

  >>> normal_club = component.getUtility(IClub)
  >>> IClub.providedBy(normal_club)
  True
  >>> isinstance(normal_club, NormalClub)
  True

Named utilities are registered using grok.name:

  >>> huge_club = component.getUtility(IClub, name='huge')
  >>> IClub.providedBy(huge_club)
  True
  >>> isinstance(huge_club, HugeClub)
  True

A utility can explicitly specify which interface it should be looked up with.

  >>> spiky_club = component.getUtility(IClub, name='spiky')
  >>> isinstance(spiky_club, SpikyClub)
  True

  >>> component.getUtility(ISpikyClub, name='spiky')
  Traceback (most recent call last):
    ...
  ComponentLookupError: (<InterfaceClass grok.tests.utility.utility.ISpikyClub>,
                         'spiky')

If a utility implements more than one interface, it has to specify the one to
use with 'grok.provides':

  >>> nightclub = component.getUtility(INightClub)
  >>> INightClub.providedBy(nightclub)
  True
  >>> isinstance(nightclub, NightClub)
  True

You can make the class the utility by providing the grok.direct() directive,
if you also use interface.classProvides instead of grok.provides.
This is useful for utilities that do nothing but create instances:

  >>> clubmaker = component.getUtility(IClubMaker, 'maker')
  >>> IClubMaker.providedBy(clubmaker)
  True
  >>> clubmaker is ClubMaker
  True

Utilities (including classes that do not subclass from grok.GlobalUtility) can
be (re-)registered using grok.global_utility:

  >>> fireplace = component.getUtility(IFireplace)
  >>> IFireplace.providedBy(fireplace)
  True
  >>> isinstance(fireplace, Fireplace)
  True

  >>> fireplace = component.getUtility(IFireplace, name='hot')
  >>> IFireplace.providedBy(fireplace)
  True
  >>> isinstance(fireplace, Fireplace)
  True

  >>> home = component.getUtility(IHome)
  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home)
  True

  >>> night = component.getUtility(INightClub, name='cool')
  >>> IClub.providedBy(night)
  True
  >>> isinstance(night, NightClub)
  True
  
  >>> spiky = component.getUtility(ISpikyClub)
  >>> ISpikyClub.providedBy(spiky)
  True
  >>> isinstance(spiky, NightClub)
  True

When re-registering a grok.GlobalUtility, the directives grok.name and
grok.provides on the class will be used, but can be overriden in the
grok.global_utility directive:

  >>> small = component.getUtility(ISmallClub, name='tiny')
  >>> ISmallClub.providedBy(small)
  True
  >>> isinstance(small, SmallClub)
  True

  >>> small2 = component.getUtility(ITinyClub, name='tiny')
  >>> ISmallClub.providedBy(small2)
  True
  >>> isinstance(small2, SmallClub)
  True
  >>> small is not small2
  True

  >>> small3 = component.getUtility(ISmallClub, name='small')
  >>> ISmallClub.providedBy(small3)
  True
  >>> isinstance(small3, SmallClub)
  True
  >>> small3 is not small2 and small3 is not small
  True

Normally one registers a utility factory, such as the class, as a
global utility. It is also possible to register an arbitrary object directly
as a global utility. You do this by passing a 'direct' argument set to
'True'. This can be useful if one needs to register functions (such
as factory functions) that can be looked up as a utility, or if the
class you want to register as a global utility has an __init__ that
takes arguments, where you want to do the instantiation yourself.
Let's look up an instance we registered this way:

  >>> small4 = component.getUtility(ISmallClub, name='smallish')
  >>> ISmallClub.providedBy(small4)
  True
  >>> isinstance(small4, SmallClub)
  True
  
"""

import grok
from zope import interface

class IClub(interface.Interface):
    pass

class ISpikyClub(IClub):
    pass

class ISmallClub(IClub):
    pass

class ITinyClub(IClub):
    pass

class INightClub(interface.Interface):
    pass

class IClubMaker(interface.Interface):
    pass

class NormalClub(grok.GlobalUtility):
    grok.implements(IClub)

class HugeClub(grok.GlobalUtility):
    grok.implements(IClub)
    grok.name('huge')    

class SpikyClub(grok.GlobalUtility):
    grok.implements(ISpikyClub)
    grok.provides(IClub)
    grok.name('spiky')

class NightClub(grok.GlobalUtility):
    grok.implements(INightClub, ISpikyClub)
    grok.provides(INightClub)

class SmallClub(grok.GlobalUtility):
    grok.implements(ISmallClub, ITinyClub)
    grok.provides(ISmallClub)
    grok.name('tiny')

class ClubMaker(grok.GlobalUtility):
    grok.implements(IClub)
    interface.classProvides(IClubMaker)
    grok.direct()
    grok.name('maker')

class IFireplace(interface.Interface):
    pass

class IHome(interface.Interface):
    pass

class Fireplace(object):
    grok.implements(IFireplace)

class Home(object):
    grok.implements(IFireplace, IHome)

grok.global_utility(Fireplace)
grok.global_utility(Fireplace, name='hot')
grok.global_utility(Home, provides=IHome)

grok.global_utility(NightClub, name='cool')
grok.global_utility(NightClub, provides=ISpikyClub)

grok.global_utility(SmallClub, provides=ITinyClub)
grok.global_utility(SmallClub, name='small')

grok.global_utility(SmallClub(), name='smallish',
                    direct=True)



##############################################################################
#
# Copyright (c) 2006-2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Grok interfaces
"""
from zope import interface
from zope.interface.interfaces import IInterface
from zope.component.interfaces import IObjectEvent
from zope.publisher.interfaces.http import IHTTPRequest
from zope.container.interfaces import IContainer as IContainerBase

# Expose interfaces from grokcore.* packages as well:
import grokcore.annotation.interfaces
import grokcore.catalog.interfaces
import grokcore.component.interfaces
import grokcore.formlib.interfaces
import grokcore.json.interfaces
import grokcore.layout.interfaces
import grokcore.rest.interfaces
import grokcore.security.interfaces
import grokcore.site.interfaces
import grokcore.traverser.interfaces
import grokcore.view.interfaces
import grokcore.viewlet.interfaces
import grokcore.xmlrpc.interfaces

from grokcore.component.interfaces import IContext
from grokcore.component.interfaces import IGrokErrors

from grokcore.rest.interfaces import IREST, IRESTSkinType, IRESTLayer

class IGrokBaseClasses(
    grokcore.annotation.interfaces.IBaseClasses,
    grokcore.catalog.interfaces.IBaseClasses,
    grokcore.component.interfaces.IBaseClasses,
    grokcore.json.interfaces.IBaseClasses,
    grokcore.layout.interfaces.IBaseClasses,
    grokcore.rest.interfaces.IBaseClasses,
    grokcore.security.interfaces.IBaseClasses,
    grokcore.site.interfaces.IBaseClasses,
    grokcore.traverser.interfaces.IBaseClasses,
    grokcore.view.interfaces.IBaseClasses,
    grokcore.xmlrpc.interfaces.IBaseClasses,
    ):
    Container = interface.Attribute(
        "Base class for containers.")

    ExceptionView = interface.Attribute(
        "Base class for excetion views.")

    Model = interface.Attribute(
        "Base class for persistent content objects (models).")

    NotFoundView = interface.Attribute(
        "Base class notfound exception views.")

    OrderedContainer = interface.Attribute(
        "Base class for ordered containers.")

    UnauthorizedView = interface.Attribute(
        "Base class unauthorized exception views.")

    View = interface.Attribute(
        "Base class views.")


class IGrokDirectives(
    grokcore.component.interfaces.IDirectives,
    grokcore.security.interfaces.IDirectives,
    grokcore.site.interfaces.IDirectives,
    grokcore.view.interfaces.IDirectives,
    ):
    pass


class IGrokEvents(interface.Interface):

    IObjectCreatedEvent = interface.Attribute("")

    ObjectCreatedEvent = interface.Attribute("")

    IObjectModifiedEvent = interface.Attribute("")

    ObjectModifiedEvent = interface.Attribute("")

    IObjectCopiedEvent = interface.Attribute("")

    ObjectCopiedEvent = interface.Attribute("")

    IObjectAddedEvent = interface.Attribute("")

    ObjectAddedEvent = interface.Attribute("")

    IObjectMovedEvent = interface.Attribute("")

    ObjectMovedEvent = interface.Attribute("")

    IObjectRemovedEvent = interface.Attribute("")

    ObjectRemovedEvent = interface.Attribute("")

    IContainerModifiedEvent = interface.Attribute("")

    ContainerModifiedEvent = interface.Attribute("")

    IBeforeTraverseEvent = interface.Attribute("")

    IApplicationInitializedEvent = interface.Attribute("")

    ApplicationInitializedEvent = interface.Attribute("")


class IGrokAPI(
    grokcore.component.interfaces.IGrokcoreComponentAPI,
    grokcore.formlib.interfaces.IGrokcoreFormlibAPI,
    grokcore.layout.interfaces.IGrokcoreLayoutAPI,
    grokcore.security.interfaces.IGrokcoreSecurityAPI,
    grokcore.site.interfaces.IGrokcoreSiteAPI,
    grokcore.view.interfaces.IGrokcoreViewAPI,
    grokcore.rest.interfaces.IGrokcoreRestAPI,
    grokcore.viewlet.interfaces.IGrokcoreViewletAPI,
    IGrokBaseClasses,
    IGrokDirectives,
    IGrokErrors,
    IGrokEvents,
    ):

    # BBB this is deprecated
    def grok(dotted_name):
        """Grok a module or package specified by ``dotted_name``.

        NOTE: This function will be removed from the public Grok
        public API.  For tests and interpreter sessions, use
        grok.testing.grok().
        """

    # BBB this is deprecated
    def grok_component(name, component, context=None, module_info=None,
                       templates=None):
        """Grok an arbitrary object. Can be useful during testing.

        name - the name of the component (class name, or global instance name
               as it would appear in a module).
        component - the object (class, etc) to grok.
        context - the context object (optional).
        module_info - the module being grokked (optional).
        templates - the templates registry (optional).

        Note that context, module_info and templates might be required
        for some grokkers which rely on them.

        NOTE: This function will be removed from the public Grok
        public API.  For tests and interpreter sessions, use
        grok.testing.grok_component().
        """

    def notify(event):
        """Send ``event`` to event subscribers."""


class IGrokView(grokcore.view.interfaces.IGrokView):
    """Grok views all provide this interface."""

    def application_url(name=None):
        """Return the URL of the closest application object in the
        hierarchy or the URL of a named object (``name`` parameter)
        relative to the closest application object.
        """

    def flash(message, type='message'):
        """Send a short message to the user."""


class IGrokForm(grokcore.formlib.interfaces.IGrokForm, IGrokView):
    """All Grok forms provides this interface."""


class IContainer(IContext, IContainerBase):
    """A Grok container.
    """



##############################################################################
#
# Copyright (c) 2006 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""The grok demo wiki
"""

import grok
import grokwiki.page

class Wiki(grok.Application, grok.Container):
    """This is Grok's sample wiki application."""

class Index(grok.View):
    def render(self):
        self.redirect(self.url('home'))

class Hello(grok.View):
    grok.baseclass()
    
    def render(self):
        return "Hello"

class Hoi(Hello):
    def render(self):
        return "Hoi"

class Bonjour(Hello):
    def render(self):
        return "Bonjour"

@grok.subscribe(Wiki, grok.IObjectAddedEvent)
def setupHomepage(wiki, event):
    """Creates a home page for every wiki."""
    page = grokwiki.page.WikiPage()
    wiki['home'] = page



"""
Multi-Adapters are supported by subclassing grok.MultiAdapter, giving
multiple arguments to grok.adapts, and supplying a matching
__init__():

  >>> grok.testing.grok(__name__)

  >>> cave = Cave()
  >>> fireplace = Fireplace()

  >>> from zope import component
  >>> home = component.getMultiAdapter((cave, fireplace))

  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home)
  True
  >>> home.cave is cave
  True
  >>> home.fireplace is fireplace
  True

This also works for named adapters using grok.name:

  >>> home = component.getMultiAdapter((cave, fireplace), name='home2')

  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home2)
  True
  >>> home.cave is cave
  True
  >>> home.fireplace is fireplace
  True

Multiadapters that implement more than one interface can use grok.provides to
specify the one to use:

  >>> home = component.getMultiAdapter((cave, fireplace), name='home3')

  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home3)
  True
  >>> home.cave is cave
  True
  >>> home.fireplace is fireplace
  True
"""

import grok
from zope import interface

class Cave(grok.Model):
    pass

class Fireplace(grok.Model):
    pass

class IHome(interface.Interface):
    pass

class Home(grok.MultiAdapter):
    grok.adapts(Cave, Fireplace)
    grok.implements(IHome)

    def __init__(self, cave, fireplace):
        self.cave = cave
        self.fireplace = fireplace

class Home2(grok.MultiAdapter):
    grok.adapts(Cave, Fireplace)
    grok.implements(IHome)
    grok.name('home2')

    def __init__(self, cave, fireplace):
        self.cave = cave
        self.fireplace = fireplace

class IFireplace(interface.Interface):
    pass

class Home3(grok.MultiAdapter):
    grok.adapts(Cave, Fireplace)
    grok.implements(IHome, IFireplace)
    grok.provides(IHome)
    grok.name('home3')

    def __init__(self, cave, fireplace):
        self.cave = cave
        self.fireplace = fireplace



import re
import unittest, doctest
import grok

from pkg_resources import resource_listdir
from zope.testing import renormalizing
from zope.app.wsgi.testlayer import BrowserLayer, http

FunctionalLayer = BrowserLayer(grok)

checker = renormalizing.RENormalizing([
    # Accommodate to exception wrapping in newer versions of mechanize
    (re.compile(r'httperror_seek_wrapper:', re.M), 'HTTPError:'),
    ])

def http_call(method, path, data=None, **kw):
    """Function to help make RESTful calls.

    method - HTTP method to use
    path - testbrowser style path
    data - (body) data to submit
    kw - any request parameters
    """

    if path.startswith('http://localhost'):
        path = path[len('http://localhost'):]
    request_string = '%s %s HTTP/1.1\n' % (method, path)
    for key, value in kw.items():
        request_string += '%s: %s\n' % (key, value)
    if data is not None:
        request_string += '\r\n'
        request_string += data
    return http(request_string, handle_errors=False)

def suiteFromPackage(name):
    files = resource_listdir(__name__, name)
    suite = unittest.TestSuite()
    for filename in files:
        if not filename.endswith('.py'):
            continue
        if filename == '__init__.py':
            continue
        dottedname = 'grok.ftests.%s.%s' % (name, filename[:-3])
        test = doctest.DocTestSuite(
            dottedname,
            checker=checker,
            extraglobs=dict(
                http_call=http_call,
                http=http,
                getRootFolder=FunctionalLayer.getRootFolder),
            optionflags=(
                doctest.ELLIPSIS+
                doctest.NORMALIZE_WHITESPACE+
                doctest.REPORT_NDIFF)
                )
        test.layer = FunctionalLayer
        suite.addTest(test)
    return suite

def test_suite():
    suite = unittest.TestSuite()
    for name in [
        'catalog',
        'chameleon',
        'errorviews',
        'form',
        'forms',
        'lifecycle',
        'security',
        'site',
        'traversal',
        'url',
        'viewlet',
        ]:
        suite.addTest(suiteFromPackage(name))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



import re
import unittest
import doctest
import zope.component.eventtesting
import zope.component.testlayer
import grok
import grok.testing

from pkg_resources import resource_listdir
from zope.testing import cleanup, renormalizing
from grokcore.view.templatereg import file_template_registry

class GrokTestLayer(zope.component.testlayer.LayerBase):
    def testSetUp(self):
        grok.testing.grok()
        zope.component.eventtesting.setUp()
        file_template_registry.ignore_templates('^\.')

    def testTearDown(self):
        cleanup.cleanUp()

layer = GrokTestLayer(grok, name='grok.tests.layer')

checker = renormalizing.RENormalizing([
    # str(Exception) has changed from Python 2.4 to 2.5 (due to
    # Exception now being a new-style class).  This changes the way
    # exceptions appear in traceback printouts.
    (re.compile(r"ConfigurationExecutionError: <class '([\w.]+)'>:"),
                r'ConfigurationExecutionError: \1:'),
    ])

def suiteFromPackage(name):
    files = resource_listdir(__name__, name)
    suite = unittest.TestSuite()
    for filename in files:
        if not filename.endswith('.py'):
            continue
        if filename.endswith('_fixture.py'):
            continue
        if filename == '__init__.py':
            continue
        dottedname = 'grok.tests.%s.%s' % (name, filename[:-3])
        test = doctest.DocTestSuite(
            dottedname,
            checker=checker,
            optionflags=doctest.ELLIPSIS+doctest.NORMALIZE_WHITESPACE)
        test.layer = layer
        suite.addTest(test)
    return suite

def test_suite():
    suite = unittest.TestSuite()
    for name in [
        'adapter',
        'baseclass',
        'conflict',
        'container',
        'directive',
        'error',
        'event',
        'grokker',
        'security',
        'traversal',
        'utility',
        'viewlet',
        'xmlrpc',
        'zcml',
        ]:
        suite.addTest(suiteFromPackage(name))
    return suite

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')



"""
When a GrokImportError occurs, ZCML will give the proper stack trace:

  >>> import grok
  >>> from zope.configuration import xmlconfig
  >>> context = xmlconfig.file('meta.zcml', grok)

  >>> ignored = xmlconfig.string('''
  ... <configure
  ...     xmlns="http://namespaces.zope.org/zope"
  ...     xmlns:grok="http://namespaces.zope.org/grok"
  ...     >
  ...     <grok:grok package="grok.tests.zcml.directiveimporterror_fixture"/>
  ... </configure>''', context=context)
  Traceback (most recent call last):
    ...
  ZopeXMLConfigurationError: File "...", line ...
  GrokImportError: The 'template' directive can only be used on class level.
"""



"""
  >>> from grokcore.layout import ILayout
  >>> from zope.component import getMultiAdapter
  >>> from zope.publisher.browser import TestRequest

  >>> cow = Cow()
  >>> request = TestRequest()

  >>> mylayout = getMultiAdapter((request, cow), ILayout)

  FormPage:
  >>> myform = getMultiAdapter((cow, request), name='myformpage')
  >>> print myform()
  <html>
   <body>
     <div class="layout"><form action="http://127.0.0.1" method="post"
        class="edit-form" enctype="multipart/form-data">
  <BLANKLINE>
  <BLANKLINE>
  <BLANKLINE>
  <BLANKLINE>
  <BLANKLINE>
    <table class="form-fields">
      <tbody>
  <BLANKLINE>
          <tr>
            <td class="label">
  <BLANKLINE>
              <label for="form.color">
                <span class="required">*</span><span>Color</span>
              </label>
            </td>
            <td class="field">
              <div class="widget"><input class="textType" id="form.color" name="form.color" size="20" type="text" value=""  /></div>
  <BLANKLINE>
            </td>
          </tr>
  <BLANKLINE>
      </tbody>
    </table>
  <BLANKLINE>
    <div id="actionsView">
  <BLANKLINE>
    </div>
  </form>
  </div>
   </body>
  </html>




  Display form:
  >>> myview = getMultiAdapter((cow, request), name='myview')
  >>> print myview()
  <html>
   <body>
     <div class="layout">...
          <tr class="even">
            <td class="fieldname">
              Color
            </td>
            <td>
              globally dark
            </td>
          </tr>...
     </div>
   </body>
  </html>

  >>> myview
  <grok.ftests.forms.forms.MyView object at ...>
  >>> myview.layout
  <grok.ftests.forms.forms.Master object at ...>
  >>> print myview.content()
   <table class="listing">
    <thead>
      <tr>
        <th class="label-column">&nbsp;</th>
        <th>&nbsp;</th>
      </tr>
    </thead>
    <tbody>
  <BLANKLINE>
        <tr class="even">
          <td class="fieldname">
            Color
          </td>
          <td>
            globally dark
          </td>
        </tr>
  <BLANKLINE>
    </tbody>
    <tfoot>
      <tr class="controls">
        <td colspan="2" class="align-right">
        </td>
      </tr>
    </tfoot>
  </table>
  <BLANKLINE>

  Edit form:
  >>> myeditview = getMultiAdapter((cow, request), name='myeditview')
  >>> print myeditview()
  <html>
   <body>
     <div class="layout"><form action="http://127.0.0.1" method="post"
        class="edit-form" enctype="multipart/form-data">
  <BLANKLINE>
  <BLANKLINE>
  <BLANKLINE>
  <BLANKLINE>
  <BLANKLINE>
    <table class="form-fields">
      <tbody>
  <BLANKLINE>
          <tr>
            <td class="label">
  <BLANKLINE>
              <label for="form.color">
                <span class="required">*</span><span>Color</span>
              </label>
            </td>
            <td class="field">
              <div class="widget"><input class="textType" id="form.color" name="form.color" size="20" type="text" value="globally dark"  /></div>
  <BLANKLINE>
            </td>
          </tr>
  <BLANKLINE>
      </tbody>
    </table>
  <BLANKLINE>
    <div id="actionsView">
      <span class="actionButtons">
        <input type="submit" id="form.actions.apply" name="form.actions.apply" value="Apply" class="button" />
      </span>
    </div>
  </form>
  </div>
   </body>
  </html>
  <BLANKLINE>

  >>> myeditview
  <grok.ftests.forms.forms.MyEditView object at ...>
  >>> myeditview.layout
  <grok.ftests.forms.forms.Master object at ...>
  >>> print myeditview.content()
  <form action="http://127.0.0.1" method="post"
        class="edit-form" enctype="multipart/form-data">
     ...<span>Color</span>...
     ... value="globally dark" ...
     ... value="Apply" ...
  </form>


"""
import grokcore.component as grok

from grokcore.view import templatedir
from grok import Layout, DisplayFormPage, EditFormPage, FormPage
from zope import interface, schema


templatedir('templates')


class ICowProperties(interface.Interface):
    color = schema.TextLine(title=u"Color")


class Cow(grok.Context):
    grok.implements(ICowProperties)
    color = u"globally dark"


class Master(Layout):
    grok.name('master')
    grok.context(Cow)


class MyView(DisplayFormPage):
    grok.context(Cow)


class MyEditView(EditFormPage):
    grok.context(Cow)

class MyFormPage(FormPage):
    grok.context(Cow)



"""
A traverser can set the __parent__ (and __name__) attributes itself,
in which case Grok's traverser won't interfere:

  >>> getRootFolder()["herd"] = Herd('The Big Mammoth Herd')

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False
  >>> browser.open("http://localhost/herd/manfred")
  >>> print browser.contents
  <html>
  <body>
  <h1>Hello, Manfred!</h1>
  <p>Manfred is part of The Three Stooges.</p>
  </body>
  </html>

  >>> browser.open("http://localhost/herd/ellie")
  >>> print browser.contents
  <html>
  <body>
  <h1>Hello, Ellie!</h1>
  <p>Ellie is part of The Three Stooges.</p>
  </body>
  </html>

"""
import grok

class Herd(grok.Model):

    def __init__(self, name):
        self.name = name

class HerdTraverser(grok.Traverser):
    grok.context(Herd)

    def traverse(self, name):
        mammoth = Mammoth(name)
        # We pretend the mammoth is the child object of some competely
        # differnt Herd object.
        mammoth.__parent__ = Herd('The Three Stooges')
        return mammoth

class Mammoth(grok.Model):

    def __init__(self, name):
        self.name = name

grok.context(Mammoth)

class Index(grok.View):
    pass

index = grok.PageTemplate("""\
<html>
<body>
<h1>Hello, <span tal:replace="context/name/title" />!</h1>
<p><span tal:replace="context/name/title" /> is part of <span tal:replace="context/__parent__/name" />.</p>
</body>
</html>
""")



"""
Default Group Behaviour
-----------------------

Grok provides a default security policy.
Here we proofe that the basic stuff will work
as expected.

Note we defined the user and the permission in the ftesting.zcml

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()

If we try to acces a public site without authentication
we will get the following goups 'zope.Anybody' and 'zope
Everybody'

  >>> browser.open("http://localhost/@@publicview")
  >>> 'zope.Anybody' in browser.contents
  True
  >>> 'zope.Everybody' in browser.contents
  True

If we try to acces a protect view by an anonyoums user
we will get an Unauthorized Message.

  >>> browser.open("http://localhost/@@protectedview")
  Traceback (most recent call last):
  ...
  HTTPError: HTTP Error 401: Unauthorized

If access the view with an authenticated request we should
get the groups zope.Authenticated.

  >>> browser.addHeader('Authorization', 'Basic foo:secret')
  >>> browser.open("http://localhost/@@publicview")
  >>> 'zope.Authenticated' in browser.contents
  True

And of course you can access the protected view.

  >>> browser.open("http://localhost/@@protectedview")
  >>> 'zope.Authenticated' in browser.contents
  True
"""

import grok
import zope.interface


class PublicView(grok.View):

    grok.context(zope.interface.Interface)
    grok.require('zope.Public')

    def render(self):
        return ', '.join(self.request.principal.groups)


class ProtectedView(grok.View):
    grok.context(zope.interface.Interface)
    grok.require('grok.test')

    def render(self):
        return ', '.join(self.request.principal.groups)



"""

We expect this grok to fail, and give 

  >>> try:
  ...     grok.testing.grok(__name__)
  ... except ConfigurationExecutionError, error:
  ...     pass
  >>> error.evalue.component
  <class 'grok.tests.error.error.CavePainting'>

"""

import grok
from zope.configuration.config import ConfigurationExecutionError

class Mammoth(grok.Model):
    pass

class CavePainting(grok.View):
    grok.template("a")

a = grok.PageTemplate("a")
cavepainting = grok.PageTemplate("b")



##############################################################################
#
# Copyright (c) 2006-2007 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Base classes for Grok application components.

When an application developer builds a Grok-based application, the
classes they define each typically inherit from one of the base classes
provided here.

"""
import simplejson
import os.path
import zope.location
import zope.errorview.browser
import zope.component
from zope.container.interfaces import IReadContainer
from zope import component
from zope import interface
from zope.interface.common.interfaces import IException
from zope.publisher.browser import BrowserPage
from zope.publisher.defaultview import getDefaultViewName
from zope.publisher.interfaces.browser import IBrowserPublisher
from zope.publisher.interfaces.http import IHTTPRequest
from zope.publisher.interfaces import INotFound
from zope.publisher.interfaces import NotFound
from zope.publisher.publish import mapply
from zope.security.interfaces import IUnauthorized

import grok
import martian.util

import grokcore.view
import grokcore.site
import grokcore.message
import grokcore.layout
import grokcore.layout.components
from grok import interfaces, util

# BBB this is for import backward compatibility.
from grokcore.xmlrpc import XMLRPC
from grokcore.rest import REST
from grokcore.json import JSON
from grokcore.content import Container
from grokcore.content import Model
from grokcore.content import OrderedContainer
from grokcore.site import Application


class ViewSupportMixin(object):

    def application_url(self, name=None, data=None):
        """Return the URL of the closest :class:`grok.Application` object in
        the hierarchy or the URL of a named object (``name``
        parameter) relative to the closest application object.
        """
        return util.application_url(self.request, self.context, name, data)

    def flash(self, message, type='message'):
        """Send a short message to the user."""
        grokcore.message.send(message, type=type, name='session')


class View(ViewSupportMixin, grokcore.view.View):
    """The base class for views with templates in Grok applications.

    Implements the :class:`grokcore.view.interfaces.IGrokView`
    interface.

    Each class that inherits from `grok.View` is designed to "render" a
    category of content objects by reducing them to a document (often an
    HTML document).  Every view has a name, and is invoked when users
    visit the URL of an eligible context object followed by the name of
    the view itself::

        http://example.com/app/folder/object/viewname

    If the view name might conflict with actual content inside of the
    context (in the above URL, the context might already contain an
    attribute or item named ``viewname``), then the URL can be explicit
    that it is asking for a view by preceding its name with ``@@``::

        http://example.com/app/folder/object/@@viewname

    Instead of returning a full document, views are sometimes used to
    provide only a snippet of information for inclusion in some larger
    document; the view can then be called from inside of another view's
    page template::

        <li tal:content="context/@@viewname">snippet goes here</li>

    A view class can specify the category of objects that it can render
    by calling the `grok.context()` directive with either a class or an
    interface.  Otherwise, Grok will attempt to determine the context
    automatically by searching the view's module for exactly one
    `grok.Model` or `grok.Container` class (or some other class
    providing the interface `IContext`) and using that class, if found.

    Grok normally creates a view's name (the name used in URLs) by
    downcasing the name of the view class itself.  The developer can
    override this by supplying the `grok.name()` directive instead.

    The view name ``index`` is special (this works whether the view
    class itself is named ``Index``, or whether ``grok.name('index')``
    is used instead).  A view named ``index`` is used to render an
    object when the user visits its URL without appending a view name.

    Each view needs to generate and return a document. There are two
    ways of doing so: either the view can provide a `render()` method
    that returns a document, or the view can be associated with a page
    template that Grok will.  Page templates can be associated with a
    view in three different ways:

    * Grok will automatically associate a view with a page template
      defined in an accompanying ``templates`` directory.  If a view
      class ``MammothList`` occurs in a module ``<src>/animal.py``, for
      example, then Grok will look for a page template with the name
      ``<src>/animal_templates/mammothlist.pt``, where ``.pt`` can be
      any page-template extension recognized by Grok.

    * Grok will automatically associate a view with a page template
      object in the same module whose name is the downcased name of the
      view class itself.  For example, a view ``MammothList`` might be
      defined in a module alongside an actual template instance named
      ``mammothlist``.

    * The developer can explicitly define the path to the page template
      file by providing the ``grok.template()`` directive.

    Before a page template is rendered, Grok will call the `update()`
    method on the view, if one is supplied, which can pre-compute values
    that the template will need to display.  Both `render()` methods and
    `update()` methods will find the context for which the view is being
    rendered under ``self.context``.

    """
    grok.baseclass()
    interface.implements(interfaces.IGrokView)

    def application_url(self, name=None, data=None):
        """Return the URL of the closest :class:`grok.Application` object in
        the hierarchy or the URL of a named object (``name``
        parameter) relative to the closest application object.
        """
        return util.application_url(self.request, self.context, name, data)

    def flash(self, message, type='message'):
        """Send a short message to the user."""
        grokcore.message.send(message, type=type, name='session')


class ExceptionView(View, zope.errorview.browser.ExceptionView):
    """Base class for rendering views for uncaught exceptions that occur during
    the application run-time and are not otherwise rendered.

    Note that when this class in not subclassed, the default error view from
    zope.errorview is being rendered.
    """
    grok.baseclass()
    grok.context(IException)
    grok.name('index')

    def update(self):
        return zope.errorview.browser.ExceptionView.update(self)

    def render(self):
        """An error view can either be rendered by an associated template, or
        it can implement this method to render itself from Python. This is
        useful if the view's output isn't XML/HTML but something computed in
        Python (plain text, PDF, etc.)

        Contrary to regular views, render() does *not* accept any parameters.
        """
        return zope.errorview.browser.ExceptionView.render(self)

    render.base_method = True


class NotFoundView(View, zope.errorview.browser.NotFoundView):
    """Base class for rendering views for INotFound exceptions.

    Note that when this class in not subclassed, the default error view from
    zope.errorview is being rendered.
    """
    grok.baseclass()
    grok.context(INotFound)
    grok.name('index')

    def update(self):
        return zope.errorview.browser.NotFoundView.update(self)

    def render(self):
        """An error view can either be rendered by an associated template, or
        it can implement this method to render itself from Python. This is
        useful if the view's output isn't XML/HTML but something computed in
        Python (plain text, PDF, etc.)

        Contrary to regular views, render() does *not* accept any parameters.
        """
        return zope.errorview.browser.NotFoundView.render(self)

    render.base_method = True


class UnauthorizedView(View, zope.errorview.browser.UnauthorizedView):
    """Base class for rendering views for IUnauthorized exceptions.

    Note that when this class in not subclassed, the default error view from
    zope.errorview is being rendered.
    """
    grok.baseclass()
    grok.context(IUnauthorized)
    grok.name('index')

    def update(self):
        return zope.errorview.browser.UnauthorizedView.update(self)

    def render(self):
        """An error view can either be rendered by an associated template, or
        it can implement this method to render itself from Python. This is
        useful if the view's output isn't XML/HTML but something computed in
        Python (plain text, PDF, etc.)

        Contrary to regular views, render() does *not* accept any parameters.
        """
        return zope.errorview.browser.UnauthorizedView.render(self)

    render.base_method = True


class Form(ViewSupportMixin, grokcore.formlib.Form):
    """The base class for forms in Grok applications.

    A class that inherits from :class:`grok.Form` is a
    :class:`grok.View` whose template will be given information about
    the fields in its context, and use that information to render an
    HTML form for adding or editing the form.  Generally developers
    use one of the subclasses:

    * :class:`grok.AddForm`
    * :class:`grok.DisplayForm`
    * :class:`grok.EditForm`

    """
    grok.baseclass()
    interface.implements(interfaces.IGrokForm)


class AddForm(ViewSupportMixin, grokcore.formlib.AddForm):
    """Base class for add forms in Grok applications."""
    grok.baseclass()
    interface.implements(interfaces.IGrokForm)


class DisplayForm(ViewSupportMixin, grokcore.formlib.DisplayForm):
    """Base class for display forms in Grok applications."""
    grok.baseclass()
    interface.implements(interfaces.IGrokForm)


class EditForm(ViewSupportMixin, grokcore.formlib.EditForm):
    """Base class for edit forms in Grok applications."""
    grok.baseclass()
    interface.implements(interfaces.IGrokForm)


class Layout(ViewSupportMixin, grokcore.layout.Layout):
    grok.baseclass()

class Page(ViewSupportMixin, grokcore.layout.Page):
    grok.baseclass()

# Default forms for form without the html and body tags
default_form_template = grokcore.view.PageTemplateFile(
    os.path.join('templates', 'default_edit_form.pt'))

default_form_template.__grok_name__ = 'default_edit_form'

default_display_template = grokcore.view.PageTemplateFile(
    os.path.join('templates', 'default_display_form.pt'))

default_display_template.__grok_name__ = 'default_display_form'

class LayoutAwareFormPage(grokcore.layout.components.LayoutAware):
    """A mixin to make form aware of layouts.
    """
    def __call__(self):
        """Calls update and returns the layout template which calls render.
        """
        mapply(self.update, (), self.request)
        if self.request.response.getStatus() in (302, 303):
            # A redirect was triggered somewhere in update().  Don't
            # continue rendering the template or doing anything else.
            return
        # update_form() is what make a layout-aware form different from
        # 'regular" layout-aware component.
        self.update_form()
        if self.request.response.getStatus() in (302, 303):
            return
        self.layout = zope.component.getMultiAdapter(
            (self.request, self.context), grokcore.layout.ILayout)
        return self.layout(self)


class FormPage(
    ViewSupportMixin,
    LayoutAwareFormPage,
    grokcore.formlib.Form
    ):
    """A form base class.
    """
    grok.baseclass()
    template = default_form_template


class AddFormPage(
    ViewSupportMixin,
    LayoutAwareFormPage,
    grokcore.formlib.AddForm
    ):
    """Base add form.
    """
    grok.baseclass()
    template = default_form_template


class EditFormPage(
    ViewSupportMixin,
    LayoutAwareFormPage,
    grokcore.formlib.EditForm
    ):
    """Base edit form.
    """
    grok.baseclass()
    template = default_form_template


class DisplayFormPage(
    ViewSupportMixin,
    LayoutAwareFormPage,
    grokcore.formlib.DisplayForm
    ):
    """Base display form.
    """
    grok.baseclass()
    template = default_display_template



"""
  >>> grok.testing.grok(__name__)

  >>> cave = Cave()
  >>> home = IHome(cave)

  >>> IHome.providedBy(home)
  True
  >>> isinstance(home, Home)
  True

  >>> fireplace = IFireplace(cave)
  >>> IFireplace.providedBy(fireplace)
  True
  >>> isinstance(fireplace, Fireplace)
  True
"""

import grok
from zope import interface

class Cave(grok.Model):
    pass

class IHome(interface.Interface):
    pass

class Home(grok.Adapter):
    grok.implements(IHome)

class IFireplace(interface.Interface):
    pass

class Fireplace(grok.Adapter):
    grok.implements(IFireplace, IHome)
    grok.provides(IFireplace)



from setuptools import setup, find_packages
import os

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

long_description = (
    read('README.txt')
    + '\n' +
    read('CHANGES.txt')
    + '\n' +
    'Download\n'
    '********\n'
    )

tests_require = [
    'zope.app.wsgi',
    'zope.configuration',
    'zope.testing',
    ]

setup(
    name='grok',
    version='1.10.2',
    author='Grok Team',
    author_email='grok-dev@zope.org',
    url='http://grok.zope.org',
    download_url='http://cheeseshop.python.org/pypi/grok/',
    description='Grok: Now even cavemen can use Zope 3!',
    long_description=long_description,
    license='ZPL',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Zope Public License',
        'Programming Language :: Python',
        'Framework :: Zope3',
        ],
    packages=find_packages('src'),
    package_dir = {'': 'src'},
    include_package_data = True,
    zip_safe=False,
    install_requires=[
        'grokcore.annotation >= 1.1',
        'grokcore.catalog',
        'grokcore.chameleon >= 1.0',
        'grokcore.component >= 2.5',
        'grokcore.content >= 1.1',
        'grokcore.formlib >= 1.4',
        'grokcore.json >= 1.2',
        'grokcore.layout >= 1.5',
        'grokcore.message',
        'grokcore.rest >= 1.3',
        'grokcore.security[role] >= 1.6',
        'grokcore.site >= 1.6.1',
        'grokcore.traverser >= 1.1',
        'grokcore.view >= 2.7',
        'grokcore.viewlet >= 1.10',
        'grokcore.view [security_publication]',
        'grokcore.xmlrpc >= 1.2',
        'martian >= 0.14',
        'pytz',
        'setuptools',
        'simplejson',
        'z3c.autoinclude',
        'zc.catalog',
        'ZODB3',
        'zope.annotation',
        'zope.app.appsetup',
        'zope.app.publication',
        'zope.app.wsgi',
        'zope.browserpage',
        'zope.catalog',
        'zope.component',
        'zope.container',
        'zope.contentprovider',
        'zope.errorview [browser]',
        'zope.event',
        'zope.exceptions',
        'zope.i18n',
        'zope.i18nmessageid',
        'zope.interface',
        'zope.intid',
        'zope.keyreference',
        'zope.lifecycleevent',
        'zope.location',
        'zope.login',
        'zope.password',
        'zope.principalregistry',
        'zope.publisher',
        'zope.schema',
        'zope.security',
        'zope.securitypolicy',
        'zope.site',
        'zope.traversing',
        ],
    tests_require=tests_require,
    extras_require={'test': tests_require},
)



"""
We make sure we indeed have a view for BoneBag; this way we know it's
registered as the default context object:


    >>> grok.testing.grok(__name__)
    >>> bag = BoneBag()
    >>> from zope import component
    >>> from zope.publisher.browser import TestRequest
    >>> request = TestRequest()
    >>> view = component.getMultiAdapter((bag, request), name='index')
    >>> view()
    'Hello world'
"""
import grok

class BoneBag(grok.Container):
    pass

class Index(grok.View):
    """A simple view to test whether BoneBag is really registered as a model.
    """
    def render(self):
        return "Hello world"



"""

By default, the views rendered for error situations is handled by
zope.errorviews::

  >>> from zope.component import getMultiAdapter
  >>> from zope.publisher.browser import TestRequest
  >>> view = getMultiAdapter((Exception(), TestRequest()), name='index')
  >>> print view()
  A system error occurred.

Error views no longer by default provide ISystemErrorView. It would result in
duplicate log output otherwise.

  >>> from zope.browser.interfaces import ISystemErrorView
  >>> ISystemErrorView.providedBy(view)
  False

  >>> from zope.publisher.interfaces import NotFound
  >>> request = TestRequest()
  >>> error = NotFound(object(), request)
  >>> view = getMultiAdapter((error, request), name='index')
  >>> print view()
  The requested resource can not be found.

  >>> from zope.security.interfaces import Unauthorized
  >>> request = TestRequest()
  >>> request.setPrincipal(MockPrincipal())
  >>> view = getMultiAdapter((Unauthorized(), request), name='index')
  >>> print view()
  Access to the requested resource is forbidden.

The default views can be selectively overridden in your application::

  >>> from grok import ExceptionView
  >>> class MyExceptionView(ExceptionView):
  ...     def render(self):
  ...         return u'This is my idea of an exception view.'
  >>> from grok.testing import grok_component
  >>> grok_component('MyExceptionView', MyExceptionView)
  True

  >>> view = getMultiAdapter((Exception(), TestRequest()), name='index')
  >>> print view()
  This is my idea of an exception view.

  >>> from grok import NotFoundView
  >>> class MyNotFoundView(NotFoundView):
  ...     def render(self):
  ...         return u'This is my idea of a not found view.'
  >>> grok_component('MyNotFoundView', MyNotFoundView)
  True

  >>> request = TestRequest()
  >>> error = NotFound(object(), request)
  >>> view = getMultiAdapter((error, request), name='index')
  >>> print view()
  This is my idea of a not found view.

  >>> from grok import UnauthorizedView
  >>> class MyUnauthorizedView(UnauthorizedView):
  ...     def render(self):
  ...         return u'This is my idea of an unauthorized view.'
  >>> grok_component('MyUnauthorizedView', MyUnauthorizedView)
  True

  >>> request = TestRequest()
  >>> request.setPrincipal(MockPrincipal())
  >>> view = getMultiAdapter((Unauthorized(), request), name='index')
  >>> print view()
  This is my idea of an unauthorized view.

  >>> class WithTemplate(ExceptionView):
  ...     grok.template('exceptionview_template')
  >>> grok_component('WithTemplate', WithTemplate)
  True

  >>> view = getMultiAdapter((Exception(), TestRequest()), name='index')
  >>> print view()
  <html>
  <body>
  <h1>Something went wrong!</h1>
  <p>Exception()</p>
  </body>
  </html>

"""
import grok

class MockPrincipal(object):
    id = 'mockprincipal'

exceptionview_template = grok.PageTemplate("""\
<html>
<body>
<h1>Something went wrong!</h1>
<p tal:content="python: repr(context)"/>
</body>
</html>
""")



"""
Containers can have an explicit traverser associated with them.  The
behaviour falls back to basic container traversal if the 'traverse'
method returns None. Normal behaviour also means that the standard
Zope 3 paradigm"items before views" is supported in the fallback.

  >>> getRootFolder()["herd"] = herd = Herd()
  >>> herd['manfred'] = Mammoth('Manfred')
  >>> herd['ellie'] = Mammoth('Ellie')

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()
  >>> browser.handleErrors = False

When we look up 'manfred', we'll get the Mammoth object as expected:

  >>> browser.open("http://localhost/herd/manfred")
  >>> print browser.contents
  Hello Manfred

When we look up 'ellie', we also get a Mammoth object and not the
Ellie view:

  >>> browser.open("http://localhost/herd/ellie")
  >>> print browser.contents
  Hello Ellie

We can, of course, get to the Ellie view explicitly:

  >>> browser.open("http://localhost/herd/@@ellie")
  >>> print browser.contents
  Hi, it's me, the Ellie view!

"""
import grok

class Herd(grok.Container):
    pass

class Traverser(grok.Traverser):
    grok.context(Herd)

    def traverse(self, name):
        # we don't really need to do anything here as we want to test
        # the fallback behaviour
        pass

class Ellie(grok.View):
    grok.context(Herd)
    grok.name('ellie')

    def render(self):
        return "Hi, it's me, the Ellie view!"

class Mammoth(grok.Model):
    def __init__(self, name):
        self.name = name

class MammothIndex(grok.View):
    grok.context(Mammoth)
    grok.name('index')

    def render(self):
        return "Hello " + self.context.name.title()



"""
You can subscribe to events using the @grok.subscribe decorator:

  >>> grok.testing.grok(__name__)
  >>> manfred = Mammoth('Manfred')
  >>> grok.notify(grok.ObjectCreatedEvent(manfred))
  >>> mammoths
  ['Manfred']
  >>> mammoths2
  ['Manfred']
  
The decorated event handling function can also be called directly:  
  
  >>> mammothAdded(Mammoth('Max'),None)
  >>> mammoths
  ['Manfred', 'Max']

"""
import grok

class Mammoth(object):
    def __init__(self, name):
        self.name = name

mammoths = []
mammoths2 = []

@grok.subscribe(Mammoth, grok.IObjectCreatedEvent)
def mammothAdded(mammoth, event):
    mammoths.append(mammoth.name)

@grok.subscribe(Mammoth, grok.ObjectCreatedEvent)
def mammothAddedInstance(mammoth, event):
    mammoths2.append(mammoth.name)



"""
====================
Application creation
====================

The application creation can be handled by Grok. In order to setup a
functional application, the creation method will go through all the
needed events : ObjectCreated, ObjectAdded, ApplicationInitialized.

Let's create our environment. We are at the zope instance root folder.

  >>> root = getRootFolder()

In this folder, we want our Cave applicate. To create it, Grok
provides a convenient function called `create_application`::

  >>> import grok.util
  >>> app = grok.util.create_application(Cave, root, 'mycave')
  Cave <zope.lifecycleevent.ObjectCreatedEvent object at ...>
  Cave <zope.lifecycleevent.ObjectAddedEvent object at ...>
  Cave <grokcore.site.interfaces.ApplicationInitializedEvent object at ...>

As we can see, the events are effectively trigged, and in the right
order. The function returns the persisted application.

  >>> print app
  <grok.ftests.lifecycle.create_application.Cave object at ...>
  >>> print app.__parent__
  <zope.site.folder.Folder object at ...>

However, if an error occurs during the creation process, the exception
is not caught by `create_application`.

In the case we provide an id that already exists, the exception will
be raised *BEFORE* the application instanciation. For this reason, and
intentionally, no event will be trigged.

  >>> app = grok.util.create_application(Cave, root, 'mycave')
  Traceback (most recent call last):
  ...
  KeyError: 'mycave'

Please note that the `create_application` function will only accept
factories implementing IApplication::

  >>> james = grok.util.create_application(Mammoth, root, 'james')
  Traceback (most recent call last):
  ...
  WrongType: <class 'grok.ftests.lifecycle.create_application.Mammoth'>

"""
import grok


class Mammoth(grok.Model):
    """A furry creature with tusks.
    """
    pass

class Cave(grok.Container, grok.Application):
    """A shelter for homeless cavemen.
    """
    pass

@grok.subscribe(Cave, grok.IObjectCreatedEvent)
@grok.subscribe(Cave, grok.IObjectAddedEvent)
@grok.subscribe(Cave, grok.IApplicationInitializedEvent)
def EventPrinter(application, event):
    print application.__class__.__name__, event



"""
Viewing a protected view with insufficient privileges will yield
Unauthorized:

  >>> from zope.app.wsgi.testlayer import Browser
  >>> browser = Browser()

  >>> browser.open("http://localhost/@@cavepainting")
  Traceback (most recent call last):
  HTTPError: HTTP Error 401: Unauthorized

  >>> browser.open("http://localhost/@@editcavepainting")
  Traceback (most recent call last):
  HTTPError: HTTP Error 401: Unauthorized

  >>> browser.open("http://localhost/@@erasecavepainting")
  Traceback (most recent call last):
  HTTPError: HTTP Error 401: Unauthorized

Let's now grant anonymous the PaintingOwner role locally (so that we
don't have to modify the global setup).  Then we can access the views
just fine:

  >>> from zope.securitypolicy.interfaces import IPrincipalRoleManager
  >>> root = getRootFolder()
  >>> IPrincipalRoleManager(root).assignRoleToPrincipal(
  ...    'paint.PaintingOwner', 'zope.anybody')

  >>> browser.open("http://localhost/@@cavepainting")
  >>> print browser.contents
  What a beautiful painting.

  >>> browser.open("http://localhost/@@editcavepainting")
  >>> print browser.contents
  Let's make it even prettier.

  >>> browser.open("http://localhost/@@erasecavepainting")
  >>> print browser.contents
  Oops, mistake, let's erase it.

  >>> browser.open("http://localhost/@@approvecavepainting")
  Traceback (most recent call last):
  HTTPError: HTTP Error 401: Unauthorized
"""

import grok
import zope.interface

class ViewPermission(grok.Permission):
    grok.name('paint.ViewPainting')

class EditPermission(grok.Permission):
    grok.name('paint.EditPainting')

class ErasePermission(grok.Permission):
    grok.name('paint.ErasePainting')

class ApprovePermission(grok.Permission):
    grok.name('paint.ApprovePainting')

class PaintingOwner(grok.Role):
    grok.name('paint.PaintingOwner')
    grok.title('Painting Owner')
    grok.permissions(
        'paint.ViewPainting', 'paint.EditPainting', 'paint.ErasePainting')

class CavePainting(grok.View):

    grok.context(zope.interface.Interface)
    grok.require(ViewPermission)

    def render(self):
        return 'What a beautiful painting.'

class EditCavePainting(grok.View):

    grok.context(zope.interface.Interface)
    grok.require(EditPermission)

    def render(self):
        return 'Let\'s make it even prettier.'

class EraseCavePainting(grok.View):

    grok.context(zope.interface.Interface)
    grok.require(ErasePermission)

    def render(self):
        return 'Oops, mistake, let\'s erase it.'

class ApproveCavePainting(grok.View):

    grok.context(zope.interface.Interface)
    grok.require(ApprovePermission)

    def render(self):
        return 'Painting owners cannot approve their paintings.'



"""
A role has to have a name to be defined.

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
  GrokError: A role needs to have a dotted name for its id.
  Use grok.name to specify one.
"""

import grok
import zope.interface

class MissingName(grok.Role):
    pass



"""
When there are two or more viewletmanagers available in the module,
a viewlet will not auto-associate but instead raise an error.

  >>> grok.testing.grok(__name__)
  Traceback (most recent call last):
    ...
  GrokError: Multiple possible viewletmanagers for
  <class 'grok.tests.viewlet.viewlet_ambiguous_manager.Viewlet'>, please use
  the 'viewletmanager' directive.

"""

import grok
from zope.interface import Interface

class ViewletManager(grok.ViewletManager):
    grok.name('foo')
    grok.context(Interface)

class ViewletManager2(grok.ViewletManager):
    grok.name('bar')
    grok.context(Interface)

class Viewlet(grok.Viewlet):
    grok.context(Interface)

    def render(self):
        return "Render method"


